package de.joerg.diary.backend.fileManagement;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import de.joerg.diary.backend.exceptions.EntryAlreadyExistsException;
import de.joerg.diary.backend.exceptions.FileForThisUserInvalidException;
import de.joerg.diary.backend.exceptions.FileInvalidException;
import de.joerg.diary.backend.tools.Crypting;
import de.joerg.diary.backend.tools.SDServices;
import de.joerg.diary.backend.tools.UTF8Helper;
import de.joerg.diary.shared.DiaryLogger;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.SDSharedServices;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRow;
import de.joerg.diary.shared.models.DiaryEvent;
import de.joerg.diary.shared.models.SDPicture;
import de.joerg.diary.shared.models.User;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Klasse zur Realisierung der Zugriffe auf DiaryEvents und Organisation dieser.
 * 
 * @author Andreas Joerg
 */
public class DiaryEventFileManager 
{
	private final String SLASH = "\\";
	
	/**
	 * Methode um ein uebergebenes DiaryEvent zu loeschen.
	 * Rueckgabe true wenn es zu diesem Datum kein einziges Ereigniss mehr gibt
	 * false wenn es noch min. 1 Ereigniss an diesem Tag gibt.
	 * 
	 * @param filename
	 * @param diaryEvent
	 * @param user
	 * @param savePath
	 * @return true/false
	 * @throws Exception
	 */
	public boolean deleteDiaryEvent(String filename, DiaryEvent diaryEvent, User user, String savePath) throws Exception
	{
		DiaryEvent evt = loadDiaryEventFromFile(savePath, filename, user);
		
		//Wenn File kein ParentFile ist
		if(evt.getSubEvents()==null||evt.getSubEvents().size()==0)
		{
			new File(savePath+SLASH+filename).delete();
			//True bedeutet, dass es zu diesem Datum definitiv keine Ereignisse mehr gibt.
			return true;
		}
		else
		{
			//Wenn ParentEvent geloescht werden soll
			if(evt.isEqual(diaryEvent))
			{
				//Neues DiaryEvent = erstes SubEvent des zu loeschenden ParentEvents
				DiaryEvent newParent = evt.getSubEvent(0);
				evt.deleteSubEvent(newParent);
				
				//Wenn das alte ParentEvent noch SubEvents hat -> fuege diese alle dem neuen ParentEvent
				//hinzu
				if(evt.getSubEvents().size()!=0)
				{
					while(evt.getSubEvents().size()!=0)
					{
						newParent.addSubEvent(evt.getSubEvent(0));
						evt.deleteSubEvent(evt.getSubEvent(0));
					}
				}
				//altes ParentEvent hat keine SubEvents mehr -> loeschen der Datei aus dem
				//ParentEvents - File
				else
				{
					doRemarkFileAsParent(savePath, filename);
				}
				writeDiaryEventToFile(newParent, filename, user, savePath);
				evt = null;
			}
			//Bei Loeschen eines SubEvents
			else
			{
				//Loeschen des SubEvents und neuschreiben des ParentEvents auf das Filesystem
				evt.deleteSubEvent(diaryEvent);
				writeDiaryEventToFile(evt, filename, user, savePath);
				
				//Wenn das geloeschte SubEvent das einzige war -> loeschen der Datei aus dem
				//ParentEvents - File
				if(evt.getSubEvents().size()==0)
				{
					doRemarkFileAsParent(savePath, filename);
				}
			}
			//false -> Es gibt noch min. 1 Ereigniss an diesem Tag
			return false;
		}
	}
	
	/**
	 * Methode um ein DiaryEvent vom FS zu loeschen.
	 * ACHTUNG!: Hier wird das komplette DiaryEventFile geloescht ohne Ruecksicht auf Verluste!
	 * Desweiteren wird kein moeglicher Eintrag im parentObjectsFile entfernt!
	 * 
	 * @param savePath
	 * @param filename
	 * @throws Exception 
	 */
	public void deleteDiaryEventFileFromFS(String savePath, String filename) throws Exception
	{
		deleteDiaryEventFileFromFS(new File(savePath+SLASH+filename));
	}
	
	/**
	 * Loescht die uebergebene Datei vom FS.
	 * 
	 * @param file
	 * @throws Exception
	 */
	public void deleteDiaryEventFileFromFS(File file) throws Exception
	{
		file.delete();
	}
	
	/**
	 * Nennt eine ungueltige Datei um. In Dateiname.inv
	 * 
	 * @param savePath
	 * @param filename
	 */
	public void markFileAsInvalid(String savePath, String filename)
	{
		new File(savePath+SLASH+filename).renameTo(new File(savePath+SLASH+filename+".inv"));
	}
	
	/**
	 * Methode zur Aenderung eines bestehenden DiaryEvents
	 * Uebergeben werden muessen das alte und neue DiaryEvent
	 * 
	 * @param oldDiaryEvent
	 * @param newDiaryEvent
	 * @param user
	 * @param savePath
	 * @throws Exception
	 */
	public void changeDiaryEvent(DiaryEvent oldDiaryEvent, DiaryEvent newDiaryEvt, User user, 
			String savePath) throws Exception
	{	
		//Nur mit Kopie des Models arbeiten
		DiaryEvent newDiaryEvent = newDiaryEvt.getCopy();
		
		String fileName = SDSharedServices.getFileName(user.getUserID(), oldDiaryEvent.getDateOfEvent());
		
		//ParentObject
		if(loadParentObjectsFile(savePath).contains(fileName))
		{	
			//Laden des bisherigen gespeicherten DiaryEvents
			DiaryEvent orgDiaryEvent = loadDiaryEventFromFile(savePath, fileName, user);

			//Wenn das alte Event dem geladenen entspricht -> ParentEvent
			if(oldDiaryEvent.isEqual(orgDiaryEvent))
			{
				//Keine Datumsaenderung
				if(newDiaryEvent.getDateOfEvent().equals(orgDiaryEvent.getDateOfEvent()))
				{
					//Prueft auf Dubletten -> Evtl Exception
					doCheckForDuplicates(orgDiaryEvent, newDiaryEvent);
					
					//SubEvents wieder hinzufuegen
					for(int x = 0; x<oldDiaryEvent.getSubEvents().size();x++)
					{
						newDiaryEvent.addSubEvent(oldDiaryEvent.getSubEvent(x));
					}
					
					writeDiaryEventToFile(newDiaryEvent, fileName, user, savePath);
				}
				//Datum wurde geaendert
				else
				{
					//Alle Untereintraege entfernen, da diese sonst mit neu an anderem Datum gespeichert werden.
					newDiaryEvent.doRemoveAllSubEvents();
					
					//Versucht den Eintrag zu aendern und prueft dabei auf Dubletten
					doChangeEntryWithDuplicatesCheck(savePath, fileName, user, oldDiaryEvent, newDiaryEvent);
				}
			}
			//-> SubEvent
			else
			{			
				//geht alle SubEvents durch
				for(int x = 0; x<orgDiaryEvent.getSubEvents().size();x++)
				{
					//-> Wenn gesuchtes gefunden wurde
					if(orgDiaryEvent.getSubEvent(x).equals(oldDiaryEvent))
					{
						//Keine Datumsaenderung
						if(newDiaryEvent.getDateOfEvent().equals(orgDiaryEvent.getDateOfEvent()))
						{
							//Prueft auf Dubletten -> Evtl Exception
							doCheckForDuplicates(orgDiaryEvent, newDiaryEvent);
							
							//Setzt die Parameter neu und beendet anschliessend die Schleife
							orgDiaryEvent.getSubEvent(x).setPlace(newDiaryEvent.getPlace());
							orgDiaryEvent.getSubEvent(x).setTimeEnd(newDiaryEvent.getTimeEnd());
							orgDiaryEvent.getSubEvent(x).setTimeStart(newDiaryEvent.getTimeStart());
							orgDiaryEvent.getSubEvent(x).setDescription(newDiaryEvent.getDescription());
							orgDiaryEvent.getSubEvent(x).setPictures(newDiaryEvent.getPictures());
							orgDiaryEvent.getSubEvent(x).setKeywords(newDiaryEvent.getKeywords());
						
							//Neuschreiben des DiaryEventFiles
							writeDiaryEventToFile(orgDiaryEvent, fileName, user, savePath);
						}
						//Datum wurde geaendert
						else
						{
							//Versucht den Eintrag zu aendern und prueft dabei auf Dubletten.
							doChangeEntryWithDuplicatesCheck(savePath, fileName, user, 
									oldDiaryEvent, newDiaryEvent);
						}
						break;
					}
				}
			}
		}
		//keinParent/einzelnes Event -> Loescht das alte File und speichert das neue, geaenderte 
		//Event anschliessend wieder ab
		else
		{			
			//Keine Datumsaenderung
			if(oldDiaryEvent.getDateOfEvent().equals(newDiaryEvent.getDateOfEvent()))
			{
				//Auf Dublette pruefen
				doCheckForDuplicates(oldDiaryEvent, newDiaryEvent);
				
				writeDiaryEventToFile(newDiaryEvent, fileName, user, savePath);
			}
			//Datum wurde geaendert
			else
			{
				//Versucht den Eintrag zu aendern und prueft dabei auf Dubletten
				doChangeEntryWithDuplicatesCheck(savePath, fileName, user, oldDiaryEvent, newDiaryEvent);
			}
		}
	}
	
	/**
	 * Schreibt ein uebergebenes DiaryEvent auf das FileSystem.
	 * (Serialisierung in XML, Verschluesselung, Speichern auf FS)
	 * Prueft ausserdem, ob die in DiaryEvent gesetzten Strings UTF-8 konform sind.
	 * 
	 * @param diaryEvent
	 * @param file
	 * @param user
	 * @param savePath
	 * @throws Exception 
	 */
	public void writeDiaryEventToFile(DiaryEvent diaryEvent, String file, User user, String savePath) throws Exception
	{
		UTF8Helper.replaceSpecialSignsInDiaryEventProps(diaryEvent);
		
		//Pruefen ob ungueltige Zeichen vorhanden sind.
		checkIfUTF8Valid(diaryEvent);
		
		ByteArrayOutputStream xmlToByteArray = new ByteArrayOutputStream(); 
		XMLEncoder xml = new XMLEncoder(xmlToByteArray);
		String content = null;
		
		xml.writeObject(diaryEvent);
		xml.close();
		
		content = xmlToByteArray.toString();
		
		content = Crypting.crypt(content, user.getUserName()+user.getUserPW());
		
		try 
		{
			FileWriter fileWriter = new FileWriter(savePath + SLASH + file);
			fileWriter.write(content);
			fileWriter.close();
		} 
		catch (IOException ex) 
		{
			//Fehler beim Schreiben des Eintrages auf das Filesystem
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0134"));
			errMsg.start();
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw new Exception(IRessources.res.getString("0134"));
		}
	}

	/**
	 * Prueft die Attribute des uebergebenen Eintrags (inkl. Untereintraege) auf ungueltige Zeichen.
	 * Falls eines gefunden wird -> Exception
	 * 
	 * @param diaryEvent
	 * @throws Exception
	 */
	private void checkIfUTF8Valid(DiaryEvent diaryEvent) throws Exception 
	{
		//Konstante fuer Methodennamen (fuer Logging im Fehlerfall)
		String METHOD = "checkIfUTF8Valid";
		
		String responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(diaryEvent.getDescription());
		//Ungueltiges Zeichen in Beschreibung
		if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
		{
			DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
					IRessources.res.getString("0125")+responeOfIsStringUTF8Valid, getClass().getName(), 
					METHOD));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0125")+responeOfIsStringUTF8Valid+
					IRessources.res.getString("0127"));
		}
		
		responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(diaryEvent.getPlace());
		//Ungueltiges Zeichen in Ort
		if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
		{
			DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
					IRessources.res.getString("0128")+responeOfIsStringUTF8Valid, getClass().getName(), 
					METHOD));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0128")+responeOfIsStringUTF8Valid+
					IRessources.res.getString("0127"));
		}
		
		responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(diaryEvent.getTimeEnd());
		//Ungueltiges Zeichen in Zeit bis
		if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
		{
			DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
					IRessources.res.getString("0129")+responeOfIsStringUTF8Valid, getClass().getName(), 
					METHOD));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0129")+responeOfIsStringUTF8Valid+
					IRessources.res.getString("0127"));
		}
		
		responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(diaryEvent.getTimeStart());
		//Ungueltiges Zeichen in Zeit von
		if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
		{
			DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
					IRessources.res.getString("0130")+responeOfIsStringUTF8Valid, getClass().getName(), 
					METHOD));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0130")+responeOfIsStringUTF8Valid+
					IRessources.res.getString("0127"));
		}

		responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(diaryEvent.getUser().getUserPW());
		//Ungueltiges Zeichen in Passwort
		if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
		{
			DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
					IRessources.res.getString("0131")+responeOfIsStringUTF8Valid, getClass().getName(), 
					METHOD));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0131")+responeOfIsStringUTF8Valid+
					IRessources.res.getString("0127"));
		}
		
		responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(diaryEvent.getUser().getUserName());
		//Ungueltiges Zeichen in Benutzername
		if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
		{
			DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
					IRessources.res.getString("0132")+responeOfIsStringUTF8Valid, getClass().getName(), 
					METHOD));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0132")+responeOfIsStringUTF8Valid+
					IRessources.res.getString("0127"));
		}
		
		responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(diaryEvent.getUser().getSavePath());
		//Ungueltiges Zeichen in Speicherpfad
		if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
		{
			DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
					IRessources.res.getString("0133")+responeOfIsStringUTF8Valid, getClass().getName(), 
					METHOD));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0133")+responeOfIsStringUTF8Valid+
					IRessources.res.getString("0127"));
		}
		
		//Pruefen auf ungueltiges Zeichen in Bildbeschreibungen
		ArrayList<SDPicture> pictures = diaryEvent.getPictures();
		if(pictures != null)
		{
			for(int x=0;x<pictures.size();x++)
			{
				if(pictures.get(x).getPictureDesc()!=null)
				{
					responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(pictures.get(x).getPictureDesc());
					if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
					{
						DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
								IRessources.res.getString("0206")+responeOfIsStringUTF8Valid, getClass().getName(), 
								METHOD));
						
						errMsg.start();
						
						throw new Exception(IRessources.res.getString("0206")+responeOfIsStringUTF8Valid+
								IRessources.res.getString("0127"));
					}
				}
			}
		}
		
		//Pruefen auf ungueltiges Zeichen in Stichwoertern
		String[] keyWords = diaryEvent.getKeywords();
		if(keyWords!=null)
		{
			for(int x=0;x<keyWords.length;x++)
			{
				if(keyWords[x]!=null)
				{
					responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(keyWords[x]);
					if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
					{
						DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
								IRessources.res.getString("0207")+responeOfIsStringUTF8Valid, getClass().getName(), 
								METHOD));
						
						errMsg.start();
						
						throw new Exception(IRessources.res.getString("0207")+responeOfIsStringUTF8Valid+
								IRessources.res.getString("0127"));
					}
				}
			}
		}
		
		//Rekursiver Aufruf, fuer ggf. vorhandene Untereintraege
		if(diaryEvent.getSubEvents()!=null)
		{
			for (DiaryEvent evt : diaryEvent.getSubEvents())
			{
				checkIfUTF8Valid(evt);
			}
		}
	}

	/**
	 * Liest ein DiaryEvent aus einer uebergebenen Datei aus und gibt es zurueck.
	 * (Einlesen von FS, Entschluesselung, Deserialiserung per XML)
	 * 
	 * @param savePath
	 * @param file
	 * @param user
	 * @return (DiaryEvent)xml.readObject()
	 * @throws Exception 
	 */
	public DiaryEvent loadDiaryEventFromFile(String savePath, String file, User user) throws Exception
	{
		String content = null;
		boolean hasUnsecureCrypting = false;
		
		//Try Block in dem der Inhalt der Datei in den String content eingelesen wird
		try 
		{
			File filePath = new File(savePath + SLASH +file);
			FileReader fileReader = new FileReader(filePath);
			char[] buffer = new char[(int)filePath.length()];
			
			fileReader.read(buffer);
			fileReader.close();
			
			content = new String(buffer);
		}
		catch (FileNotFoundException ex) 
		{
			//Zu ladener Eintrag wurde nicht gefunden
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0135") + savePath + SLASH +
					file + IRessources.res.getString("0136"));
			errMsg.start();
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw new FileNotFoundException(IRessources.res.getString("0135") + savePath + SLASH + 
					file + IRessources.res.getString("0136"));
		} 
		catch (IOException ex) 
		{
			//Fehler beim Dateizugriff bei Schreiben des Eintrages auf das Filesystem.
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0137"));
			errMsg.start();
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw new IOException(IRessources.res.getString("0137"));
		}
		
		try
		{
			//Entschluesselt den Inhalt der Datei
			content = Crypting.decrypt(content, user.getUserName()+user.getUserPW());
		}
		//Wenn der Inhalt nicht entschluesselt werden konnte
		//Grund ist evtl. dass die Datei noch mit einer aelteren Version ohne AES Verschluesselung
		//gecrypted wurde.
		catch(Exception ex)
		{
			//Datei konnte nicht entschluesselt werden, Versuch mit Crypt V1.0.0.1
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0138") + file.toString() + 
					IRessources.res.getString("0139"));
			errMsg.start();
			
			content = Crypting.decryptOld(content, user.getUserName()+user.getUserPW());
			hasUnsecureCrypting = true;
		}
			
		XMLDecoder xml = new XMLDecoder(new ByteArrayInputStream(content.getBytes()));
		
		try
		{
			//Laden des DiaryEvents mittels XML
			DiaryEvent evt = (DiaryEvent)xml.readObject();
			xml.close();
			
			//Wenn der Inhalt der Datei ueber den alten Verschluesselungs-Algorithmus entschluesselt wurde
			//-> Aufruf der Methode changeDiaryEvent(), bewirkt, dass die Datei neu geschrieben wird
			//MIT AES Verschluesselung
			if(hasUnsecureCrypting)
			{
				changeDiaryEvent(evt, evt, user, savePath);
			}
			
			//Stellt gecryptede Sonderzeichen in den Properties des DiaryEvents wieder her.
			UTF8Helper.encodeSpecialSignsInDiaryEventProps(evt);

			if(evt.getUser().equals(user))
			{
				SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT2);
				sdf.setLenient(false);
				Date date = sdf.parse(file.substring(file.indexOf("_")+1, 
						file.indexOf(".")));
				
				//Wenn das Datum im Dateinamen dem Datum des DiaryEvents entspricht
				if(date.equals(evt.getDateOfEvent()))
				{
					return evt;
				}
				//Sollte nur auftreten, falls eine Datei manuell umbenannt wird.
				else
				{
					//Datei hat falsches Datum im Dateinamen. Um lesen zu koennen umbenennen in ...
					throw new FileInvalidException(IRessources.res.getString("0138")+ file + 
							IRessources.res.getString("0140") + user.getUserID() +
							"_"+sdf.format(evt.getDateOfEvent())+IRessources.res.getString("0141"));
				}
			}
			
			//Datei ist ungueltig. PW/ID falsch
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0138") + file.toString() + 
					IRessources.res.getString("0142"));
			errMsg.start();
				
			throw new FileForThisUserInvalidException(IRessources.res.getString("0138") + file.toString() + 
					IRessources.res.getString("0142"));
		}
		catch(ParseException ex)
		{
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			//Datei hat ungueltiges Datum in Dateiname
			throw new FileInvalidException(IRessources.res.getString("0138") + file + 
					IRessources.res.getString("0143"));
		}
		catch(FileInvalidException ex)
		{
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw ex;
		}
		catch(FileForThisUserInvalidException ex)
		{
			throw ex;
		}
		catch(Exception ex)
		{
			//Datei nicht lesbar
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0138") + file.toString() + 
					IRessources.res.getString("0144"));
			errMsg.start();
			
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
				
			throw new FileInvalidException(IRessources.res.getString("0138") + file.toString() + 
					IRessources.res.getString("0144"));
		}
	
	}
	
	/**
	 * Schreibt dir Datei auf das FileSystem, welche die Filenamen enthalten, in denen DiaryEvents
	 * gespeichert sind, die SubEvents enthalten.
	 * (Serialisierung mit XML, Verschluesselung, Speicher auf FS)
	 * 
	 * @param files
	 * @param savePath
	 * @throws Exception 
	 */
	private void writeParentFile(ArrayList<String> files, String savePath) throws Exception
	{
		try 
		{
			ByteArrayOutputStream xmlToByte = new ByteArrayOutputStream();
			XMLEncoder xml = new XMLEncoder(xmlToByte);
			String content = null;
			
			xml.writeObject(files);
			xml.close();
			
			content = xmlToByte.toString();
			content = Crypting.crypt(content, "MD53%Fg344A222!__as5..(F");
			
			FileWriter writer = new FileWriter(new File(savePath+SLASH+IRessources.FILE_PARENT_ENTRYS));
			writer.write(content);
			writer.close();
		} 
		catch (IOException ex) 
		{
			//Fehler bei IO bei KZ Eintrag als Parent
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0145"));
			errMsg.start();
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw new Exception(IRessources.res.getString("0145"));
		}
	}
	
	@SuppressWarnings("unchecked")
	/**
	 * Liest die Datei ein, welche die Filenamen enthalten, in denen DiaryEvents gespeichert sind,
	 * die SubEvents enthalten.
	 * (Datei einlesen, Entschluesselung, Deserialiserung per XML, Rueckgabe der ArrayList)
	 * 
	 * @param savePath
	 */
	public ArrayList<String> loadParentObjectsFile(String savePath) throws Exception
	{
		try 
		{
			boolean hasUnsecureCrypting = false;
			//---------Liest den Inhalt der Datei in den String content ein------
			File fileParents = new File(savePath+SLASH+IRessources.FILE_PARENT_ENTRYS);
			FileReader reader = new FileReader(fileParents);
			char[] buffer = new char[(int)fileParents.length()];
			String content = null;
			
			reader.read(buffer);
			reader.close();
			
			content = new String(buffer);
			//--------------------------------------------------------------------
			
			try
			{
				//Entschluesseln des Inhalts aus der Datei
				content = Crypting.decrypt(content, "MD53%Fg344A222!__as5..(F");
			}
			//Wenn beim Entschluesseln ein Fehler aufgetreten ist
			//-> Moeglicher Grund: Datei wurde noch mit altem Verschluesselungs-Algorithmus verschluesselt
			// -> Aufruf der Methode Crypting.decryptOld() um zu versuchen den String mit dieser Methode
			//	  zu entschluesseln.
			catch(Exception ex)
			{
				//Datei konnte nicht decrypted werden. Try mit Crypto V1.0.0.1
				DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0138") + 
						fileParents.toString() + IRessources.res.getString("0139"));
				
				errMsg.start();
				
				content = Crypting.decryptOld(content, "MD53%Fg344A222!__as5..(F");
				hasUnsecureCrypting = true;
			}
			
			
			XMLDecoder xml = new XMLDecoder(new ByteArrayInputStream(content.getBytes()));
			ArrayList<String> parentObjects = (ArrayList<String>) xml.readObject();
			xml.close();
			
			//Wenn die Datei noch mit dem altem Algorithmus verschluesselt war.
			// -> Aufruf der Methode writeParentFile, bewirkt dass die Datei neu geschrieben wird
			//    MIT AES Crypting.
			if(hasUnsecureCrypting)
			{
				writeParentFile(parentObjects, savePath);
			}
			
			return parentObjects;
		} 
		catch (FileNotFoundException ex) 
		{
			//Hier kein ErrorLogging, da es sonst schnell passieren kann, dass die LogDatei mit
			//dieser Exception zugespammt wird wenn kein einziges ParentFile vorhanden ist.
			return new ArrayList<String>();
		} 
		catch (IOException ex) 
		{
			//Fehler beim IO auf Datei ..
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0146")+savePath+SLASH
					+IRessources.FILE_PARENT_ENTRYS);
			errMsg.start();
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw new Exception(IRessources.res.getString("0146")+savePath+SLASH+
					IRessources.FILE_PARENT_ENTRYS);
		}
	}
	
	/**
	 * Fuegt einen uebergebenen Dateinamen (+Pfad) zur ParentEvents Datei hinzu.
	 * 
	 * @param savePath
	 * @param file
	 * @throws Exception 
	 */
	public void markFileAsParent(String savePath, File file) throws Exception
	{
		ArrayList<String> parentObjects = loadParentObjectsFile(savePath);
		
		//Wenn der Eintrag bereits markiert ist -> return (=tue nichts)
		for (String entry : parentObjects) 
		{
			if(entry.equals(file))
			{
				return;
			}
		}
		
		parentObjects.add(file.toString());
		
		writeParentFile(parentObjects, savePath);
	}
	
	/**
	 * Entfernt einen uebergebenen Dateinamen aus dem ParentObjects File.
	 * 
	 * @param savePath
	 * @param fileName
	 * @throws Exception
	 */
	public void doRemarkFileAsParent(String savePath, String fileName) throws Exception
	{
		ArrayList<String> parentObjects = loadParentObjectsFile(savePath);
		
		parentObjects.remove(fileName);
		
		writeParentFile(parentObjects, savePath);
	}
	
	/**
	 * Speichert ein DiaryEvent auf das Filesystem.
	 * 
	 * @param diaryEvent
	 * @param user
	 * @param savePath
	 * @throws Exception 
	 */
	public void saveDiaryEvent(DiaryEvent diaryEvt, User user, String savePath) throws Exception
	{
		DiaryEvent diaryEvent = diaryEvt.getCopy();
		
		ArrayList<String> filesInPath = getListOfEventFiles(user, savePath);
		boolean isParentDiaryEventExisting = false;
		SimpleDateFormat dfDiaryEventDate = new SimpleDateFormat(IRessources.DATE_FORMAT2);
		
		for (String file : filesInPath) 
		{
			//Wenn Parentobject vorhanden ist.
			if(file.indexOf(dfDiaryEventDate.format(diaryEvent.getDateOfEvent()))!=-1)
			{
				isParentDiaryEventExisting = true;
			}
		}
		
		//Wenn bereits ein DiaryEvent unter diesem Datum gespeichert wurde ->
		//Laden des ParentEvents, hinzufuegen des aktuellen DiaryEvents zum ParentEvent, Speicherung
		//des ParentEvents auf das FileSystem, File als ParentFile markieren.
		if(isParentDiaryEventExisting)
		{
				DiaryEvent parent = loadDiaryEventFromFile(savePath,String.valueOf(user.getUserID())+
						"_"+dfDiaryEventDate.format(diaryEvent.getDateOfEvent())+IRessources.FILE_EXT, user);
			
				//Dubletten Pruefung -> Evtl. Exception
				doCheckForDuplicates(parent, diaryEvent);
				
				parent.addSubEvent(diaryEvent);
				
				writeDiaryEventToFile(parent, String.valueOf(user.getUserID())+"_"+dfDiaryEventDate.format(
						parent.getDateOfEvent())+IRessources.FILE_EXT, user, savePath);

				//Wenn Eintrag noch nicht als parent markiert ist -> markieren
				if(!loadParentObjectsFile(savePath).contains(String.valueOf(user.getUserID())+"_"+dfDiaryEventDate.format(
						parent.getDateOfEvent())+IRessources.FILE_EXT))
				{
					markFileAsParent(savePath, new File(String.valueOf(user.getUserID())+"_"+dfDiaryEventDate.format(
							parent.getDateOfEvent())+IRessources.FILE_EXT));
				}
		}
		else
		{		
			//--------- MANTIS 80 (Jahr unter 1000 Bug---------------------------------
			String checkYear = dfDiaryEventDate.format(diaryEvent.getDateOfEvent());
			checkYear = checkYear.substring(6);
			if(Integer.valueOf(checkYear)<1000)
			{
				//Ungueltiges Datum
				throw new Exception(IRessources.res.getString("0070"));
			}
			//-------------------------------------------------------------------------
			
			//Datum darf nicht in Zukunft liegen (Mantis 96)
			if(diaryEvent.getDateOfEvent().after(new Date()))
			{
				throw new Exception(IRessources.res.getString("0172"));
			}
			
			writeDiaryEventToFile(diaryEvent,String.valueOf(user.getUserID()+"_"+
					dfDiaryEventDate.format(diaryEvent.getDateOfEvent())+IRessources.FILE_EXT), user, 
					savePath);
		}
	}	
	
	/**
	 * Ueberprueft die Gueltigkeit eines uebergebnen Dateinamens nach Konvention: USERID_DD_MM_YYYY.def
	 * z.B.: 1000_11.12.2004.def
	 * 
	 * @param fileName
	 * @return fileName
	 */
	public boolean checkFileName(String fileName)
	{
		if(fileName==null)
		{
			//Dateiname darf nicht null sein
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0147")+
					getClass().getName()+" <checkFileName>");
			
			errMsg.start();
			
			throw new NullPointerException(IRessources.res.getString("0148"));
		}
		if(fileName.length()!=19)
		{
			//Keine Exception + kein Log bei .inv Dateien, die von SD selbst als ungueltig gekennzeichnet
			//wurden, sonst -> flooding des Logfiles.
			if(fileName.indexOf(".inv")==fileName.length()-4)
			{
				return false;
			}
			
			//Dateiname hat ungueltige Laenge
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0149") + fileName +
					IRessources.res.getString("0150") + getClass().getName()+" <checkFileName>");
			
			errMsg.start();
			
			throw new IllegalArgumentException(IRessources.res.getString("0149") + fileName + 
					IRessources.res.getString("0151"));
		}
		
		//49-57: 1-9
		if(!((int)fileName.charAt(0)>=49&&(int)fileName.charAt(0)<=57))
		{
			return false;
		}
		for(int x = 1;x<4;x++)
		{
			//48-57: 0-9
			if(!((int)fileName.charAt(x)>=48&&(int)fileName.charAt(x)<=57))
			{
				return false;
			}
		}
		if(fileName.charAt(4)!='_')
		{
			return false;
		}
		//48-51: 0-3
		if(!((int)fileName.charAt(5)>=48&&(int)fileName.charAt(5)<=51))
		{
			return false;
		}
		//0-9
		if(!((int)fileName.charAt(6)>=48&&(int)fileName.charAt(6)<=57))
		{
			return false;
		}
		//50: 2
		if(fileName.charAt(5)=='3'&&(int)fileName.charAt(6)>=50)
		{
			return false;
		}
		if(fileName.charAt(5)=='0'&&(int)fileName.charAt(6)=='0')
		{
			return false;
		}
		if(fileName.charAt(7)!='_')
		{
			return false;
		}
		if(fileName.charAt(8)!='0'&&fileName.charAt(8)!='1')
		{
			return false;
		}
		//0-9
		if(!((int)fileName.charAt(9)>=48&&(int)fileName.charAt(9)<=57))
		{
			return false;
		}
		//50 = 2
		if(fileName.charAt(8)=='1'&&fileName.charAt(9)>50)
		{
			return false;
		}
		if(fileName.charAt(8)=='0'&&fileName.charAt(9)=='0')
		{
			return false;
		}
		if(fileName.charAt(10)!='_')
		{
			return false;
		}
		if(fileName.charAt(11)!='1'&&fileName.charAt(11)!='2')
		{
			return false;
		}
		for(int x = 12;x<=14;x++)
		{
			if(!((int)fileName.charAt(x)>=48&&(int)fileName.charAt(x)<=57))
			{
				return false;
			}
		}
		if(!fileName.substring(15).equals(IRessources.FILE_EXT))
		{
			return false;
		}
		
		return true;
	}
	
	/**
	 * Gibt die gueltigen DiaryEvent - Files des uebergebenem Users im uebergebenem Verzeichniss zurueck.
	 * 
	 * @param user
	 * @param savePath
	 * @return diaryEventsForUserInPath
	 * @throws Exception 
	 */
	public ArrayList<String> getListOfEventFiles(User user, String savePath) throws Exception
	{
		File dir = new File(savePath);
		File[] fileList = dir.listFiles();
		ArrayList<String> diaryEventsForUserInPath = new ArrayList<String>();
		
		if(fileList == null)
		{
			//Pfad ungueltig
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0152")+savePath+
					IRessources.res.getString("0153") + getClass().getName()+" <getListOfEventFiles>");
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0152") + savePath + 
					IRessources.res.getString("0154"));
		}
		
		for (File file : fileList) 
		{
			if(file.getName().length()>String.valueOf(user.getUserID()).length())
			{
				if(file.getName().substring(0,(String.valueOf(user.getUserID())).length()).equals(String.
						valueOf(user.getUserID())))
				{
					try
					{
						if(checkFileName(file.getName()))
						{
							diaryEventsForUserInPath.add(file.getName());
						}
					}
					catch(IllegalArgumentException ex)
					{
						DiaryLogger log = new DiaryLogger(ex);
						log.start();
					}
				}
			}
		}
		
		sortListOfEventFilesByDate(diaryEventsForUserInPath);
		
		return diaryEventsForUserInPath;
	}
	
	/**
	 * Sortiert die Eintraege einer Liste aufsteigend nach dem Datum
	 * 
	 * @param evtFiles
	 * @throws Exception 
	 */
	private void sortListOfEventFilesByDate(ArrayList<String> evtFiles) throws Exception
	{		
		//BubbleSort Algorithmus
		for(int x = 0;x<evtFiles.size()-1;x++)
		{
			for(int y = 1;y<evtFiles.size();y++)
			{
				//Wenn das Datum danach ist -> tausche
				if(isDateAfter(evtFiles.get(y-1),evtFiles.get(y)))
				{
					String temp = evtFiles.get(y-1);
					evtFiles.set(y-1, evtFiles.get(y));
					evtFiles.set(y, temp);
				}
			}
		}
	}
	
	/**
	 * Die Methode bekommt zwei Strings uebergeben im Format (xxxx_dd_MM_yyyy.def), holt sich das Datum aus
	 * beiden Strings und vergleicht diese. Wenn das erste Datum nach dem zweitem Datum ist -> return true
	 * 
	 * @param date1
	 * @param date2
	 * @return true/false
	 * @throws Exception 
	 */
	private boolean isDateAfter(String date1, String date2) throws Exception
	{
		SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT2);
		try 
		{
			Date firstDate = sdf.parse(date1.substring(date1.indexOf("_")+1, date1.indexOf(".")));
			Date secondDate = sdf.parse(date2.substring(date2.indexOf("_")+1, date2.indexOf(".")));
			
			return firstDate.after(secondDate);
		} 
		catch (ParseException ex) 
		{
			//Fehler bei Date parsen
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0155"));
			errMsg.start();
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw new Exception(IRessources.res.getString("0155"));
		}
	}
	
	/**
	 * Methode die einen uebergebenen Eintrag (oldDiaryEvent) zu einem neuen Eintrag (newDiaryEvent) aendert.
	 * Dabei wird geprueft, ob der neue Eintrag bereits im alten Eintrag existiert.
	 * Wenn ja -> EntryAlreadyExistsException
	 * 
	 * @param savePath
	 * @param fileName
	 * @param user
	 * @param oldDiaryEvent
	 * @param newDiaryEvent
	 * @throws Exception
	 */
	private void doChangeEntryWithDuplicatesCheck(String savePath, String fileName, User user,
			DiaryEvent oldDiaryEvent, DiaryEvent newDiaryEvent) throws Exception
	{
		//Alter Eintrag muss zwischengespeichert werden, da er im naechsten Schritt bereits
		//geloescht wird. Falls in der Methode saveDiaryEvent eine Exception
		//auftritt muss der Eintrag wieder zuruecksetzbar sein.
		DiaryEvent tempEvt = loadDiaryEventFromFile(savePath, fileName, user);
		
		deleteDiaryEvent(fileName, oldDiaryEvent, user, savePath);
		
		//Stellt den Eintrag wieder her falls eine Exception auftritt
		try
		{
			saveDiaryEvent(newDiaryEvent, user, savePath);
		}
		catch(Exception ex)
		{
			DiaryLogger errLog = new DiaryLogger(ex);
			errLog.start();
			
			writeDiaryEventToFile(tempEvt, fileName, user, savePath);
			
			//Eintrag muss neu markiert werden als ParentFile, da er aus der Liste geloescht
			//wird falls der letzte Untereintrag zuvor geloescht wurde
			//Wird nur markiert, wenn er auch wirklich Untereintraege hat, da die Methode auch von Aenderungen
			//an einzelnen Eintraegen aufgerufen werden kann.
			if(tempEvt.getSubEvents()!=null&&tempEvt.getSubEvents().size()!=0)
			{
				markFileAsParent(savePath, new File(fileName));
			}
			
			throw ex;
		}
	}
	
	/**
	 * Methode die prueft, ob ein uebergebener Eintrag (newEvt) bereits in einem anderen Eintrag (existingEvt)
	 * vorhanden ist und in dessen Untereintraegen (falls vorhanden).
	 * Wenn ja -> EntryAlreadyExistsException
	 * 
	 * @param existingEvt
	 * @param newEvt
	 * @throws EntryAlreadyExistsException
	 */
	private void doCheckForDuplicates(DiaryEvent existingEvt, DiaryEvent newEvt) 
		throws EntryAlreadyExistsException
	{
		//Dubletten Erkennung Parent (MANTIS 38)
		if(existingEvt.isSame(newEvt))
		{
			throw new EntryAlreadyExistsException();
		}
		
		//Sind Untereintraege vorhanden?
		if(existingEvt.getSubEvents()!=null&&existingEvt.getSubEvents().size()!=0)
		{
			//Dubletten Erkennung Untereintraege
			for (DiaryEvent child : existingEvt.getSubEvents()) 
			{
				if(child.isSame(newEvt))
				{
					throw new EntryAlreadyExistsException();
				}
			}
		}
	}
	
	/**
	 * Loescht alle Eintraege an dem Tag mit dem uebergebenem Datum.
	 * 
	 * @param date
	 * @param savePath
	 * @param user
	 * @throws Exception
	 */
	public void deleteAllEntrysOnDay(String date, String savePath, User user) throws Exception
	{
		//Validierung ob uebergebenes Datum gueltig. Wenn nicht wird hier automatisch eine ParseException
		//geworfen.
		new SimpleDateFormat(IRessources.DATE_FORMAT).parse(date);
		
		//Dateinamen bilden
		String fileName = SDSharedServices.getFileName(user.getUserID(), date);
		
		//Eintrag komplett loeschen
		deleteDiaryEventFileFromFS(savePath, fileName);
		
		//KZ des Eintrags als MultiEintrag entfernen
		doRemarkFileAsParent(savePath, fileName);
	}
	
	/**
	 * Erstellt ein Backup der Eintragsdateien des uebergebenen Users im uebergebenen Verzeichnis.
	 * 
	 * @param savePath
	 * @param user
	 * @param bkpFilePath
	 * @throws Exception
	 */
	public void doBackUp(String savePath, User user, String bkpFilePath) throws Exception
	{
		//Holen Liste Dateien fuer User in Path
		ArrayList<String> filesInPath = getListOfEventFiles(user, savePath);
		
		//keien Eintraege
		if(filesInPath.size()<1)
		{
			throw new Exception(IRessources.res.getString("0230"));
		}
		
		//Holen Liste MultiEintraege
		ArrayList<String> parents = loadParentObjectsFile(savePath);
		
		//DataSet fuer Files
		DataSet dsBkpFileContent = new DataSet();
		
		//Dateiendung .bkp anhaengen
		bkpFilePath += IRessources.BKP_FILE_EXT;
		
		//Schleife fuer alle Eintragsdateien im Pfad
		for (String fileName : filesInPath)
		{
			boolean isMulti = false;
			
			File file = new File(savePath+SLASH+fileName);
			
			char buffer[] = new char[(int) file.length()];
			
			//Eintrag einlesen
			FileReader reader = new FileReader(file);
			reader.read(buffer);
			reader.close();
			
			//Wenn Eintrag Untereintraege hat
			for (String str : parents)
			{
				if(fileName.equals(str))
				{
					isMulti = true;
					break;
				}
			}
			
			//Multieintrag -> "!" KZ vor Dateiname setzen
			if(isMulti)
			{
				//Dateiname + Inhalt DataSet hinzufuegen
				dsBkpFileContent.addRow("!"+fileName, new String(buffer));
			}
			else
			{
				//Dateiname + Inhalt DataSet hinzufuegen
				dsBkpFileContent.addRow(fileName, new String(buffer));
			}
		}
		
		//DataSet als XML in String speichern
		ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
		XMLEncoder xml = new XMLEncoder(byteOut);
		xml.writeObject(dsBkpFileContent);
		xml.close();
		String content = byteOut.toString();
		
		//Datei verschluesselt speichern
		FileWriter write = new FileWriter(new File(bkpFilePath));
		write.write(Crypting.crypt(content, user.getUserName()+user.getUserPW()));
		write.close();
	}

	/**
	 * Stellt ein Backup im uebergebenen Verzeichnis wieder her.
	 * ACHTUNG: EINTRAEGE WERDEN GGF UEBERSCHRIEBEN
	 * 
	 * @param savePath
	 * @param bkpFile
	 * @param user
	 * @throws Exception
	 */
	public void doBackUpRedo(String savePath, String bkpFile, User user) throws Exception 
	{
		File file = new File(bkpFile);
		DataSet dataSet = null;
		char[] buffer = new char[(int) file.length()];
		
		//Datei einlesen und entschluesselt in String speichern
		FileReader read = new FileReader(file);
		read.read(buffer);
		read.close();
		String content = new String(buffer);
		content = Crypting.decrypt(content, user.getUserName()+user.getUserPW());
		
		//XML Daten in DataSet einlesen
		XMLDecoder xml = new XMLDecoder(new ByteArrayInputStream(content.getBytes()));
		dataSet = (DataSet) xml.readObject();
		xml.close();
		
		//Jede Datei wieder herstellen
		for (DataSetRow row : dataSet.getRows())
		{
			//MultiEintrag
			if(row.getAttributeName().charAt(0)=='!')
			{
				//! aus namen loeschen wegen Speicherung des Files.
				row.setAttributeName(row.getAttributeName().substring(1));
				
				//Als MultiEintrag markieren
				markFileAsParent(savePath, new File(row.getAttributeName()));
			}
			
			//Eintrag auf FS schreiben
			FileWriter writer = new FileWriter(savePath+SLASH+row.getAttributeName());
			writer.write((String)row.getAttribute());
			writer.close();
		}
	}
	
	/**
	 * Sucht nach dem uebergebenen Kriterium in allen Eintraegen
	 * 
	 * @param type
	 * @param crit
	 * @param user
	 * @return Suchergbnisse
	 * @throws Exception
	 */
	public ArrayList<String> doSearch(int type, String crit, User user) throws Exception
	{
		return doSearch(type, crit, user, false, null, null);
	}
	
	/**
	 * Sucht nach dem uebergebenem Kriterien in allen Eintraegen im gewuenschten Zeitraum (checkTimeSpan muss
	 * dazu auf true gesetzt sein.)
	 * 
	 * @param type
	 * @param crit
	 * @param user
	 * @param checkTimeSpan
	 * @param timeSpanBegin
	 * @param timeSpanEnd
	 * @return Suchergebnisse
	 * @throws Exception
	 */
	public ArrayList<String> doSearch(int type, String crit, User user, boolean checkTimeSpan,
			String timeSpanBegin, String timeSpanEnd) throws Exception
	{
		//Ungueltiger Suchtyp
		if(type!=0&&type!=1&&type!=2)
		{
			throw new Exception(IRessources.res.getString("0226"));
		}
		
		//Suchkriterium null
		if(crit==null||crit.isEmpty())
		{
			throw new Exception(IRessources.res.getString("0210"));
		}
		
		//Holen Dateien des Users
		ArrayList<String> files = getListOfEventFiles(user, user.getSavePath());
		
		//keien Dateien da
		if(files == null||files.size()==0)
		{
			throw new Exception(IRessources.res.getString("0228"));
		}
		
		//Zeitraumpruefung
		if(checkTimeSpan)
		{
			getFilesForTimeSpan(files, timeSpanBegin, timeSpanEnd);
		}
		
		//keien Dateien da
		if(files == null||files.size()==0)
		{
			throw new Exception(IRessources.res.getString("0227"));
		}
		
		//Liste zur Speicherung der Suchergebnise
		ArrayList<String> found = new ArrayList<String>();
		
		//Schleife die jeden Eintrag durchsucht
		for (String file : files) 
		{
			//Holen naechsten Eintrag
			DiaryEvent evt = loadDiaryEventFromFile(user.getSavePath(), file, user);
			
			//Stichwort
			if(type == 0)
			{
				searchForKeyword(evt, crit, found, 1);
			}
			//Volltext
			else if(type == 1)
			{
				searchForFulltext(evt, crit, found, 1);
			}
			//Ort
			else if(type == 2)
			{
				searchForPlace(evt, crit, found, 1);
			}
		}	
			
		return found;
	}

	/**
	 * Filtert aus der uebergebenen Liste die Dateien heraus, die nicht im gewuenschten Zeitraum sind.
	 * 
	 * @param files
	 * @param timeSpanBegin
	 * @param timeSpanEnd
	 * @throws Exception
	 */
	private void getFilesForTimeSpan(ArrayList<String> files, String timeSpanBegin, String timeSpanEnd) 
																throws Exception
	{
		SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
		Date from = sdf.parse(timeSpanBegin);
		Date to = sdf.parse(timeSpanEnd);
		
		//Anfangsdatum darf nicht nach dem Enddatum sein.
		if(from.after(to))
		{
			throw new Exception(IRessources.res.getString("0225"));
		}
		
		Iterator<String> it = files.iterator();
		boolean isAfter = false;
		
		//Iterieren ueber Eintragliste
		while(it.hasNext())
		{
			//Wenn Flag isAfter gesetzt ist -> Eintrag entfernen
			if(isAfter)
			{
				it.next();
				it.remove();
			}
			else
			{
				sdf = new SimpleDateFormat(IRessources.DATE_FORMAT2);

				Date actDate = sdf.parse(it.next().substring(5,15));
				
				//Wenn Datum des Eintrags vor Zeitspanne ist -> entfernen
				if(actDate.before(from))
				{
					it.remove();
				}
				//Wenn Datum des Eintrags nach Zeitspanne ist -> entfernen und setzen des Flag isAfter, um
				//alle nachfolgenden Dateieintraege zu loeschen
				else if(actDate.after(to))
				{
						isAfter = true;
						it.remove();
					
				}
			}
		}
	}

	/**
	 * Sucht in dem uebergebenem Eintrag nach dem Ort und fuegt den Eintrag ggf. zur Liste "found" hinzu.
	 * 
	 * @param evt	-> Eintrag
	 * @param crit	-> gesuchter Ort
	 * @param found	-> Liste gefunder Eintrage
	 * @param index	-> Parent- Subevent ? 1=Parent, 2=Child1, 3=Child2 ...
	 */
	private void searchForPlace(DiaryEvent evt, String crit, ArrayList<String> found, int index) 
	{
		//Wenn der Ort dem gesuchten entspricht
		if(evt.getPlace()!=null&&evt.getPlace().contains(crit))
		{
			//Wenn parent und keine Subevents da -> " - x" weglassen
			if(index==1&&(evt.getSubEvents()==null||evt.getSubEvents().size()==0))
			{
				found.add(new SimpleDateFormat(IRessources.DATE_FORMAT).format(evt.getDateOfEvent()));
			}
			else
			{
				found.add(new SimpleDateFormat(IRessources.DATE_FORMAT).format(evt.getDateOfEvent())+" - "+index);
			}
		}
		
		//Rekursiver Aufruf fuer Subevents
		if(evt.getSubEvents() != null)
		{
			for (DiaryEvent child : evt.getSubEvents())
			{
				index++;
				
				searchForPlace(child, crit, found, index);
			}
		}
	}

	/**
	 * Sucht in der Beschreibung des uebergebenen Eintrags nach dem Text und fuegt den Eintrag ggf. zur Liste 
	 * "found" hinzu.
	 * 
	 * @param evt	-> Eintrag
	 * @param crit	-> gesuchter Text
	 * @param found	-> Liste gefunder Eintrage
	 * @param index	-> Parent- Subevent ? 1=Parent, 2=Child1, 3=Child2 ...
	 */
	private void searchForFulltext(DiaryEvent evt, String crit, ArrayList<String> found, int index) 
	{
		//Wenn der Eintrag eine Beschreibung hat und diese den gesuchten Text enthaelt
		if(evt.getDescription()!=null&&evt.getDescription().contains(crit))
		{
			//Wenn parent und keine Subevents da -> " - x" weglassen
			if(index==1&&(evt.getSubEvents()==null||evt.getSubEvents().size()==0))
			{
				found.add(new SimpleDateFormat(IRessources.DATE_FORMAT).format(evt.getDateOfEvent()));
			}
			else
			{
				found.add(new SimpleDateFormat(IRessources.DATE_FORMAT).format(evt.getDateOfEvent())+" - "+index);
			}
		}
		
		//Rekursiver Aufruf fuer Subevents
		if(evt.getSubEvents() != null)
		{
			for (DiaryEvent child : evt.getSubEvents())
			{
				index++;
				
				searchForFulltext(child, crit, found, index);
			}
		}
	}

	/**
	 * Sucht in dem uebergebenem Eintrag nach dem Stichwort und fuegt den Eintrag ggf. zur Liste "found" hinzu.
	 * 
	 * @param evt	-> Eintrag
	 * @param crit	-> gesuchtes Stichwort
	 * @param found	-> Liste gefunder Eintrage
	 * @param index	-> Parent- Subevent ? 1=Parent, 2=Child1, 3=Child2 ...
	 */
	private void searchForKeyword(DiaryEvent evt, String crit, ArrayList<String> found, int index) 
	{
		//Holen Stichwoerter
		String[] keywords = evt.getKeywords();
		
		//Wenn Stichwoerter da sind
		if(keywords!=null)
		{
			//Schleife fuer jedes Stichwort
			for (String key : keywords) 
			{
				//Wenn Stichwort gesuchtem entspricht
				if(crit.equals(key))
				{
					//Wenn parent und keine Subevents da -> " - x" weglassen
					if(index==1&&(evt.getSubEvents()==null||evt.getSubEvents().size()==0))
					{
						found.add(new SimpleDateFormat(IRessources.DATE_FORMAT).format(evt.getDateOfEvent()));
					}
					else
					{
						found.add(new SimpleDateFormat(IRessources.DATE_FORMAT).format(evt.getDateOfEvent())+" - "+index);
					}
					
					break;
				}
			}
		}
		
		//Rekursiver Aufruf fuer Subevents
		if(evt.getSubEvents() != null)
		{
			for (DiaryEvent child : evt.getSubEvents())
			{
				index++;
				
				searchForKeyword(child, crit, found, index);
			}
		}
	}
}