package de.joerg.diary.frontend.sdCalendar;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import javax.swing.JLabel;
import de.joerg.diary.shared.DiaryLogger;
import de.joerg.diary.shared.IRessources;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt den Controller eines SDCalendars.
 * 
 * @author Andreas Joerg
 */
public class SDCalendar implements MouseListener
{
	private ArrayList<String> entrys = new ArrayList<String>();
	private ICalendarListener calendarListener = null;
	private SDCalendarPanel panel = null;
	private Date actuallDate = null;
	private SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
	
	/**
	 * Standard constructor.
	 * Initialisiert ein neues SDCalendarPanel mit aktuellem Datum
	 * 
	 * @throws NumberFormatException
	 * @throws ParseException
	 */
	public SDCalendar(ICalendarListener listener) throws NumberFormatException, ParseException
	{
		setPanel(new SDCalendarPanel(this));
		setActuallDate(new Date());
		
		doSetControlsForActuallMonth();
		
		setCalendarListener(listener);
	}

	/**
	 * Setzt das Calendar Panel
	 * 
	 * @param panel
	 */
	public void setPanel(SDCalendarPanel panel)
	{
		this.panel = panel;
	}
	/**
	 * Gibt das Calendar Panel zurueck.
	 * 
	 * @return panel
	 */
	public SDCalendarPanel getPanel()
	{
		return panel;
	}

	/**
	 * Setzt das aktuelle Datum (erster im Monat des uebergebenen Datums)
	 * 
	 * @param actuallDate
	 * @throws ParseException
	 */
	public void setActuallDate(Date actuallDate) throws ParseException 
	{
		String date = sdf.format(actuallDate);
		date = "01"+date.substring(2);

		this.actuallDate = sdf.parse(date);
	}
	
	/**
	 * @return aktuelles Datum
	 */
	public Date getActuallDate()
	{
		return actuallDate;
	}
	
	/**
	 * Setzt die Eintraege die vorhanden sind und geoeffnet werden koennen
	 * 
	 * @param entrys
	 */
	public void setEntrys(ArrayList<String> entrys) 
	{
		this.entrys = entrys;
	}
	/**
	 * @return Liste aller Eintraege
	 */
	public ArrayList<String> getEntrys() 
	{
		return entrys;
	}

	/**
	 * Setzt den CalendarListener, der auf Mausklicks auf Datumsfelder reagiert
	 * 
	 * @param listener
	 */
	public void setCalendarListener(ICalendarListener listener) 
	{
		this.calendarListener = listener;
	}
	/**
	 * @return CalendarListener
	 */
	public ICalendarListener getCalendarListener() 
	{
		return calendarListener;
	}
	
	/**
	 * Setzt die Labels des Panels entsprechend des aktuellen Monats (und Jahres)
	 * 
	 * @throws NumberFormatException
	 * @throws ParseException
	 */
	public void doSetControlsForActuallMonth() throws NumberFormatException, ParseException
	{
		panel.doSetControlsForMonth(
				Integer.valueOf( 
				sdf.format( getActuallDate() ).substring(3, 5) ),
				Integer.valueOf(
				sdf.format( getActuallDate() ).substring(6) ) );
	}
	
	/**
	 * Setzt die Labels des Panels entsprechend des uebergebenen Strings.
	 * Setzt ebenfalls das aktuelle Datum auf das entsprechende Datum.
	 * 
	 * @param date
	 * @throws ParseException
	 */
	public void doSetControlsForDate(String date) throws ParseException
	{
			setActuallDate(sdf.parse(date));
			panel.doCleanUp();
			doSetControlsForActuallMonth();
			doUpdate();
	}
	
	/**
	 * Entfernt alle Eintraege -> ggf Refresh Ansicht
	 * 
	 * @throws NumberFormatException
	 * @throws ParseException
	 */
	public void doRemoveAllEntrys() throws NumberFormatException, ParseException
	{
		entrys.clear();
		doSetControlsForActuallMonth();
		doUpdate();
	}
	
	/**
	 * Fuegt einen Eintrag hinzu -> kein Refresh dieser muss manuell noch mittels doUpdate ausgefuehrt werden
	 * da sonst nach jedem Hinzufuegen eines Eintrags ein Update performed werden wuerde was zu performance-
	 * lastig waere.
	 * 
	 * @param entry
	 */
	public void addEntry(String entry)
	{
		String entryName = entry.substring(entry.indexOf("_")+1, entry.lastIndexOf("."));
		entryName = entryName.replace('_', '.');
		
		if(entry.contains("!"))
		{
			entryName = "!"+entryName;
		}
		
		entrys.add(entryName);
	}
	
	/**
	 * Refresh der Ansicht zB nach Speichervorgang oder Loeschvorgang.
	 */
	public void doUpdate()
	{
		ArrayList<String> entrysForMonth = new ArrayList<String>();
		String date = sdf.format(getActuallDate()).substring(2);
		
		for (String entry : entrys)
		{
			if(entry.contains(date))
			{
				entrysForMonth.add(entry);
			}
		}
		
		getPanel().doUpdate(entrysForMonth, date);
	}
	
	/**
	 * Loest ein CalendarClickEvent aus und sendet es an den Listener.
	 * 
	 * @param date
	 * @param isMulti
	 */
	private void fireCalendarClickEvent(String date, boolean isMulti)
	{
		if(calendarListener!=null)
		{
			calendarListener.doHandleCalendarClick(new SDCalendarClickEvent(isMulti, 
					getDateFromSDLabel(date)));
		}
	}
	
	/**
	 * Loest ein CalendarClickEvent aus und sendet es an den Listener.
	 * Achtung: Mit dieser Methode werden nur Rechtsklicks behandelt!
	 * 
	 * @param date
	 * @param isMulti
	 */
	private void fireCalendarRightClickEvent(String date, boolean isMulti, int x, int y)
	{
		if(calendarListener!=null)
		{
			calendarListener.doHandleCalendarRightClick(new SDCalendarRightClickEvent(isMulti, 
					getDateFromSDLabel(date), x, y));
		}
	}
	
	private String getDateFromSDLabel(String date)
	{
		if(Integer.valueOf(date)<10)
		{
			date = "0"+date;
		}
		return date + sdf.format(getActuallDate()).substring(2);
	}
	
	/**
	 * Behandelt eine ParseException bei Klicks auf Monat/Jahr vor/zurueck -> Kann eignentlich nicht
	 * auftreten, da der String ein formatiertes Datum ist (getActuallDate)
	 * 
	 * @param ex
	 */
	public void handleParseExceptionAtChangeMonthOrYear(ParseException ex) 
	{
		DiaryLogger exLog = new DiaryLogger(ex);
		exLog.start();
	}
	
	@Override
	public void mouseClicked(MouseEvent evt) 
	{
		//Linksklick
		if(evt.getButton() == 1)
		{
			JLabel lbl = (JLabel) evt.getComponent();
			String date = sdf.format(getActuallDate());
			
			//Jahr zurueck
			if(lbl.getText().equals("<<"))
			{
				date = date.substring(0, 6) + (Integer.valueOf( date.substring(6) )-1);
				try 
				{
					doSetControlsForDate(date);
				} 
				catch (ParseException ex) 
				{
					handleParseExceptionAtChangeMonthOrYear(ex);
				}
			}
			
			//Monat zurueck
			else if(lbl.getText().equals("<"))
			{
				if((Integer.valueOf( date.substring(3,5) )-1)<10)
				{
					date = date.substring(0,3) + "0" + 
					( Integer.valueOf( date.substring(3,5) ) -1 ) + date.substring(5);
				}
				else
				{
					date = date.substring(0,3) + ( Integer.valueOf( date.substring(3,5) ) -1 ) + date.substring(5);
				}
				
				try 
				{
					doSetControlsForDate(date);
				} 
				catch (ParseException ex) 
				{
					handleParseExceptionAtChangeMonthOrYear(ex);
				}
			}
			
			//Monat vor
			else if(lbl.getText().equals(">"))
			{
				if((Integer.valueOf( date.substring(3,5) )-1)<10)
				{
					date = date.substring(0,3) + "0" + 
					( Integer.valueOf( date.substring(3,5) ) +1 ) + date.substring(5);
				}
				else
				{
					date = date.substring(0,3) + ( Integer.valueOf( date.substring(3,5) ) +1 ) + date.substring(5);
				}
				
				try 
				{
					doSetControlsForDate(date);
				} 
				catch (ParseException ex) 
				{
					handleParseExceptionAtChangeMonthOrYear(ex);
				}
			}
			
			//Jahr vor
			else if(lbl.getText().equals(">>"))
			{
				date = date.substring(0, 6) + (Integer.valueOf( date.substring(6) )+1);
				try 
				{
					doSetControlsForDate(date);
				} 
				catch (ParseException ex) 
				{
					handleParseExceptionAtChangeMonthOrYear(ex);
				}
			}
			//Klick auf einen Eintrag
			else
			{
				SDLabel sdLbl = (SDLabel)evt.getComponent();
				
				//Event in Listener ausfuehren.
				fireCalendarClickEvent(sdLbl.getText(), sdLbl.isMulti());
			}
		}
		//Rechtsklick
		else if(evt.getButton()==3&&evt.getClickCount()==1)
		{
			if(evt.getComponent() instanceof SDLabel)
			{
				SDLabel sdLbl = (SDLabel)evt.getComponent();
				
				//Event in Listener ausfueheren
				fireCalendarRightClickEvent(sdLbl.getText(), sdLbl.isMulti(), sdLbl.getX(), sdLbl.getY()+
						sdLbl.getHeight());
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent arg0) 
	{
	}

	@Override
	public void mouseExited(MouseEvent arg0) 
	{
	}

	@Override
	public void mousePressed(MouseEvent arg0) 
	{
	}

	@Override
	public void mouseReleased(MouseEvent arg0)
	{
	}
}